/*   Date : 23-2-1989    */

#include <stdio.h>
#include <stdlib.h>
#include <alloc.h>
#include <string.h>
#include <dir.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>

#define VERSION "TMPhone Version 1.00\n\n"

#define DIAL_RECORD     struct dial_records
#define FON_LEN         sizeof(DIAL_RECORD)
#define FON_SIZE        100
#define MEMO_LEN        30
#define CASE_SENSITIVE  0

#define NAME            0
#define PHONE           1
#define BAUD            2
#define TOTAL           3
#define LASTCALL        4

DIAL_RECORD {
     char name[30];          /* BBS name */
     char password[10];      /* user password */
     char f1;                /* filler */
     char script[8];         /* script file */
     char f2[2];             /* filler */
     char log[8];            /* log file */
     char f3;                /* filler */
     char phone[15];         /* phone number */
     char para[10];          /* com para: ( "%6ld%c%d%d ",Baud,Parity,Data,Stop ) */
     char term;              /* terminal type: 'A'=ANSI, 'T'=TTY */
     char lineFeed;          /* add line feed: 'Y'=Yes, 'N'=No */
     char localEcho;         /* local echo: 'Y'=Yes, 'N'=No */
     char connectTo;         /* connect to: 'M'=Modem, 'C'=Computer */
     char prefix;            /* dial prefix: 1 - 4 */
     char protocol;          /* default protocol: ZSTYGBXMRC12 */
     char port;              /* com port: 1 - 4 */
     char f4[3];             /* filler */
     char total[5];          /* number of time connected */
     char last[10];          /* last connected date */
     char crlf[2];           /* CR and LF */
} *dial_record;

char *dial_memo[FON_SIZE];

int cmp_by,ascending;

DIAL_RECORD *get_dir( int n );

int phone_cmp( DIAL_RECORD *p1,DIAL_RECORD *p2 )
{
     int i,n1,n2,asc;
     for ( i=n1=n2=0; i<30 ; i++ )
     {    if ( p1->name[i]=='-' ) n1++;
          if ( p2->name[i]=='-' ) n2++;
     }
     if ( n1>25 || n2>25 )
          return( n1-n2 );        /* blank name is placed at the bottom */

     switch ( cmp_by )
     { case NAME:
          asc = ascending;
          goto do_sort;
   name_sort:
          asc = 1;
   do_sort:
          return( asc*(CASE_SENSITIVE ? strncmp(p1->name,p2->name,30):strnicmp(p1->name,p2->name,30)) );
       case PHONE:
          for ( i=n1=n2=0 ; i<15 ; i++ )
          {    if ( p1->phone[i]=='.' ) n1++;
               if ( p2->phone[i]=='.' ) n2++;
          }
          if ( n1>10 && n2>10 )
               goto name_sort;
          if ( n1>10 || n2>10 )
               return( n1-n2 );
          return( ascending*(strncmp(p1->phone,p2->phone,30)) );
       case BAUD:
          if ( (i=strncmp(p1->para,p2->para,6))==0 )
               goto name_sort;
          return( ascending*i );
       case TOTAL:
          n1 = atoi(p1->total);
          n2 = atoi(p2->total);
          if ( n1==n2 )
               goto name_sort;
          return( ascending*(n1-n2) );
       case LASTCALL:
          if ( p1->last[2]=='.' && p2->last[2]=='.' )
               goto name_sort;
          if ( p1->last[2]=='.' ) return(  1 );
          if ( p2->last[2]=='.' ) return( -1 );
          if ( (i=strncmp(&p1->last[8],&p2->last[8],2)) == 0 )
               if ( (i=strncmp(&p1->last[5],&p2->last[5],2)) == 0 )
                    i = strncmp(&p1->last[2],&p2->last[2],2);
          if ( i==0 )
               goto name_sort;
          return( ascending*i );
     }
     return( 0 );
}

void phone_sort( int sort_by,int in_ascending_order )
{
     int i,j,m;
     char *memo;
     static DIAL_RECORD dir;
     cmp_by = sort_by;
     ascending = in_ascending_order ? 1:-1;
     for ( i=0 ; i<FON_SIZE-1 ; i++ )
     {    for ( j=(m=i)+1 ; j<FON_SIZE ; j++ )
               if ( phone_cmp(&dial_record[m],&dial_record[j])>0 )
                    m = j;
          if ( m!=i )
          {    memcpy( &dir,get_dir(m+1),FON_LEN );
               memcpy( get_dir(m+1),get_dir(i+1),FON_LEN );
               memcpy( get_dir(i+1),&dir,FON_LEN );
               memo = dial_memo[m];
               dial_memo[m] = dial_memo[i];
               dial_memo[i] = memo;
          }
     }
}

void load_dial_memo( char *name )
{
     int len,i;
     static char s[85],*sp;
     static char xmemo[MAXPATH],xdrive[MAXDRIVE],xdir[MAXDIR],
                 xname[MAXFILE],xext[MAXEXT];
     FILE *fp;

     fnsplit( name,xdrive,xdir,xname,xext );
     fnmerge( name=xmemo,xdrive,xdir,xname,".mem" );
     if ( (fp=fopen(name,"r+b")) !=NULL )
     {    while ( fgets(s,80,fp)!=NULL )
          {    sp = strtok(s,":" );
               if ( (i=atoi(sp))==0 || i>FON_SIZE )
                    break;
               i--;
               sp = strtok(NULL,"\r\0");
               if ( (len=strlen(sp))>MEMO_LEN ) len=MEMO_LEN;
               dial_memo[i] = realloc(dial_memo[i],len+1);
               memcpy(dial_memo[i],sp,len);
               dial_memo[i][len] = '\0';
          }
          fclose(fp);
     }
}

void save_dial_memo( char *name )
{
     int i;
     static char xmemo[MAXPATH],xdrive[MAXDRIVE],xdir[MAXDIR],
                 xname[MAXFILE],xext[MAXEXT];
     FILE *fp;

     for ( i=0 ; i<FON_SIZE && dial_memo[i][0]=='\0' ; i++ );
     if ( i==FON_SIZE )
          return;                      /* no memo */

     fnsplit( name,xdrive,xdir,xname,xext );
     fnmerge( name=xmemo,xdrive,xdir,xname,".mem" );
     unlink( name );
     if ( (fp=fopen(name,"w+b")) !=NULL )
     {    for ( i=0 ; i<FON_SIZE ; i++ )
               if ( dial_memo[i][0]!='\0' )
                    fprintf( fp,"%d:%s\r\n",i+1,dial_memo[i] );
          fclose(fp);
     }
}

void load_dial_dir( char *dir_name )
{
     int fd;
     fd = open( dir_name,O_RDONLY | O_BINARY );
     if ( fd==-1 )
     {    printf( "Cannot open phone directory: %s\n",dir_name );
          exit(2);
     }
     read( fd,dial_record,FON_SIZE*FON_LEN );
     close( fd );
}

void save_dial_dir( char *dir_name )
{
     int fd;
     unlink( dir_name );
     fd = open( dir_name,O_WRONLY | O_BINARY | O_CREAT, S_IREAD | S_IWRITE );
     write( fd,dial_record,FON_SIZE*FON_LEN );
     close( fd );
}

DIAL_RECORD *get_dir( int n )          /* n = 1 to 100 */
{
     return( &dial_record[n-1] );
}

void alloc_dial_dir( void )
{
     dial_record = calloc( FON_SIZE,FON_LEN );
}

void free_dial_dir( void )
{
     free( dial_record );
}

void alloc_dial_memo( void )
{
     int i;
     for ( i=0 ; i<FON_SIZE ; i++ )
          dial_memo[i] = calloc(1,1);
}

void main( int argc,char *argv[] )
{
     int c,sort_by,in_order;
     printf( VERSION );
     if ( argc!=2 )
     {    printf( "Usage: TMPhone phone_directory\n" );
          exit(1);
     }
     alloc_dial_dir();
     alloc_dial_memo();
     load_dial_dir( argv[1] );
     load_dial_memo( argv[1] );

     printf( "Sort by (1) Name\n" );
     printf( "        (2) Phone number\n" );
     printf( "        (3) Baud rate\n" );
     printf( "        (4) Total connection\n" );
     printf( "        (5) Last call date\n" );
     printf( "or      (Q) Quit\n" );
     printf( "\nWhich one ? " );
     while ( ((c=getch())<'1' || c>'5') && (c=toupper(c))!='Q' ) ;
     printf( "%c\n",c );
     if ( c!='Q' )
     {    sort_by = c-'1';
          printf( "\nIn (A) Ascending order\n" );
          printf( "   (D) Descending order\n" );
          printf( "or (Q) Quit\n" );
          printf( "\nWhich one ? " );
          while ( 1 )
          {    c = toupper( getch() );
               if ( c=='A' || c=='D' || c=='Q' )
                    break;
          }
          printf( "%c\n",c );
          if ( c!='Q' )
          {    printf( "\nSorting ... " );
               phone_sort( sort_by,c=='A' );
               printf( "\n\n(S) Save phone directory\n" );
               printf( "(Q) Quit\n" );
               printf( "\nWhich one ? " );
               while ( 1 )
               {    c = toupper( getch() );
                    if ( c=='S' || c=='Q' )
                         break;
               }
               printf( "%c\n",c );
               if ( c!='Q' )
               {    printf( "\nSaving ... " );
                    save_dial_dir( argv[1] );
                    save_dial_memo( argv[1] );
               }
          }
     }
     printf( "\nFinished\n" );
     free( dial_record );
     exit(0);
}
